#include "RandomAccessBinary.h"
#include <memory>
#include <vector>
#include <iomanip>
#include <iostream>
#include <algorithm>
namespace NPC_core {

RandomAccessBinary::RandomAccessBinary() : RandomAccessBinary(1548)
{}

RandomAccessBinary::RandomAccessBinary(uint possible_packetsize)
{
    this->ptr_data = std::make_unique<std::vector<uint8_t>>();
    this->ptr_data->reserve(possible_packetsize);
    this->endianness = RandomAccessBinary::Endianness::BIG;
}

RandomAccessBinary::~RandomAccessBinary()
{

}

RandomAccessBinary::Endianness RandomAccessBinary::getEndianness()
{
    return this->endianness;
}

void RandomAccessBinary::setEndianness(RandomAccessBinary::Endianness endianness)
{
    this->endianness = endianness;
}

void RandomAccessBinary::setBytes(uint position, const std::vector<uint8_t> &data)
{
    auto it = this->ptr_data->begin();
    for(uint i = 0; i < data.size(); ++i)
        this->ptr_data->insert(it+(position+i), data[i]);
}

void RandomAccessBinary::setByte(uint position, const uint8_t &data)
{
    auto it = this->ptr_data->begin();
    this->ptr_data->insert(it+position, data);
}

void RandomAccessBinary::setBit(uint64_t position, const bool &value)
{
    uint bytePosition = static_cast<uint>(position>>3);
    uint bitPosition = position % 8;
    this->setBit(bytePosition, bitPosition, value);
}

void RandomAccessBinary::setBit(uint bytePosition, uint bitPosition, const bool &value)
{
    if(ptr_data->size() < bytePosition+1)
        ptr_data->insert(ptr_data->begin()+bytePosition, 0x00);
    uint8_t byte = this->ptr_data->at(bytePosition);
    uint8_t newbyte = 0;
    if (value)
        newbyte = byte | (1<< (7-bitPosition));
    else {
        newbyte = byte & ~(1 << (7-bitPosition));
    }
    ptr_data->insert(ptr_data->begin()+bytePosition, newbyte);
}

void RandomAccessBinary::setBits(uint64_t position, uint bits, const uint8_t &value)
{
    uint64_t bytePosition = position >> 3;
    uint bitPosition = position % 8;
    this->setBits(bytePosition, bitPosition, bits, value);
}

void RandomAccessBinary::setBits(uint bytePosition, uint bitPosition, uint bits, const uint8_t &value)
{
    if(ptr_data->size() < bytePosition+1)
        ptr_data->insert(ptr_data->begin()+bytePosition, 0x00);
    uint8_t byte = this->ptr_data->at(bytePosition);
    uint8_t newbyte = value;
    uint8_t mask = 0;
    for(uint i = 0; i < bits; ++i)
        mask |= (1 << i);
    mask <<= (8-bits-bitPosition);
    byte &= ~mask;
    newbyte <<= (8-bits-bitPosition);
    byte |= newbyte;
    ptr_data->insert(ptr_data->begin()+bytePosition, byte);
}

void RandomAccessBinary::set_uint16(uint byteOffset, const uint16_t &data)
{
    uint8_t * conv = (uint8_t*)&data;
    std::vector<uint8_t> tmp;
    tmp.assign(conv, conv+sizeof(uint16_t));
    if (this->endianness != Endianness::LITTLE)
        std::reverse(tmp.begin(), tmp.end());
    this->setBytes(byteOffset, tmp);
}

void RandomAccessBinary::set_uint32(uint byteOffset, const uint32_t &data)
{
    uint8_t * conv = (uint8_t*)&data;
    std::vector<uint8_t> tmp;
    tmp.assign(conv, conv+sizeof(uint32_t));
    if (this->endianness != Endianness::LITTLE)
        std::reverse(tmp.begin(), tmp.end());
    this->setBytes(byteOffset, tmp);
}

void RandomAccessBinary::set_uint64(uint byteOffset, const uint64_t &data)
{
    uint8_t * conv = (uint8_t*)&data;
    std::vector<uint8_t> tmp;
    tmp.assign(conv, conv+sizeof(uint64_t));
    if (this->endianness != Endianness::LITTLE)
        std::reverse(tmp.begin(), tmp.end());
    this->setBytes(byteOffset, tmp);
}

void RandomAccessBinary::set_float(uint position, float &data)
{
    uint8_t * conv = (uint8_t*)&data;
    std::vector<uint8_t> tmp;
    tmp.assign(conv, conv+sizeof(float));
    if (this->endianness != Endianness::LITTLE)
        std::reverse(tmp.begin(), tmp.end());
    this->setBytes(position, tmp);
}

void RandomAccessBinary::set_double(uint position, double &data)
{
    uint8_t * conv = (uint8_t*)&data;
    std::vector<uint8_t> tmp;
    tmp.assign(conv, conv+sizeof(double));
    if (this->endianness != Endianness::LITTLE)
        std::reverse(tmp.begin(), tmp.end());
    this->setBytes(position, tmp);
}

uint8_t RandomAccessBinary::getByte(uint position)
{
    return this->ptr_data->at(position);
}

const std::vector<uint8_t> RandomAccessBinary::getBytes(uint position, uint length)
{
    auto begin = ptr_data->begin() + position;
    auto end = ptr_data->begin() + position + length;
    std::vector<uint8_t> vec(begin, end);
    return vec;
}

bool RandomAccessBinary::getBit(uint byteOffset, uint bitOffset)
{
    uint8_t byte = this->ptr_data->at(byteOffset);
    return (byte & (1 << (7-bitOffset))) > 0;
}

bool RandomAccessBinary::getBit(uint64_t position)
{
    uint byteOffset = static_cast<uint>(position>>3);
    uint bitOffset = position % 8;
    return this->getBit(byteOffset, bitOffset);
}

const std::vector<bool> RandomAccessBinary::getBits(uint byteOffset, uint8_t bitOffset, uint64_t length)
{
    uint64_t byteLength = length >> 3;
    int olength = length - (byteLength<<3) - bitOffset;
    std::vector<uint8_t> tmp(ptr_data->begin()+byteOffset, ptr_data->begin()+byteOffset+byteLength+1);
    std::vector<bool> output;
    uint i = 0;
    for(auto it = tmp.begin(); it != tmp.end(); ++it, ++i){
        if(i == 0){
            for(int a = 7-bitOffset; a >= 0; --a)
                output.push_back((*it & (1 << a)) > 0);
        } else if(olength > 0 && i == byteLength) {
            for(int a = 7; a >= olength; --a)
                output.push_back((*it & (1 << a)) > 0);
        } else {
            for(int a = 7; a >= 0; --a)
                output.push_back((*it & (1 << a)) > 0);
        }
    }
    return output;
}

const std::vector<bool> RandomAccessBinary::getBits(uint64_t position, uint64_t length)
{
    uint byteOffset = static_cast<uint>(position>>3);
    uint8_t bitOffset = position % 8;
    return this->getBits(byteOffset,bitOffset,length);
}

uint16_t RandomAccessBinary::get_uint16(uint byteOffset)
{
    auto tmp = getBytes(byteOffset, sizeof(uint16_t));
    if (endianness != Endianness::LITTLE)
        std::reverse(tmp.begin(), tmp.end());
    if (tmp.data() != nullptr) {
        uint16_t* d = (uint16_t*)tmp.data();
        return *d;
    }
    return 0;
}

uint32_t RandomAccessBinary::get_uint32(uint byteOffset)
{
    auto tmp = getBytes(byteOffset, sizeof(uint32_t));
    if (endianness != Endianness::LITTLE)
        std::reverse(tmp.begin(), tmp.end());
    if (tmp.data() != nullptr) {
        uint32_t* d = (uint32_t*)tmp.data();
        return *d;
    }
    return 0;
}

uint64_t RandomAccessBinary::get_uint64(uint byteOffset)
{
    auto tmp = getBytes(byteOffset, sizeof(uint64_t));
    if (endianness != Endianness::LITTLE)
        std::reverse(tmp.begin(), tmp.end());
    if (tmp.data() != nullptr) {
        uint64_t* d = (uint64_t*)tmp.data();
        return *d;
    }
    return 0;
}

float RandomAccessBinary::get_float(uint byteOffset)
{
    float tmp = 0.0;
    auto bytes = this->getBytes(byteOffset, sizeof(float));
    uint8_t* ptr_tmp = (uint8_t*)&tmp;
    if (endianness != Endianness::LITTLE) {
        std::reverse(std::begin(bytes), std::end(bytes));
    }
    uint8_t i = 0;
    for(auto it = bytes.begin(); it != bytes.end();++it, ++i)
        *(ptr_tmp+i) = *it;
    return tmp;
}

double RandomAccessBinary::get_double(uint byteOffset)
{
    double tmp = 0.0;
    auto bytes = this->getBytes(byteOffset, sizeof(double));
    uint8_t* ptr_tmp = (uint8_t*)&tmp;
    if (endianness != Endianness::LITTLE) {
        std::reverse(std::begin(bytes), std::end(bytes));
    }
    uint8_t i = 0;
    for(auto it = bytes.begin(); it != bytes.end();++it, ++i)
        *(ptr_tmp+i) = *it;
    return tmp;
}

std::__cxx11::string RandomAccessBinary::get_string(uint byteOffset, uint length)
{
    std::stringstream output;
    uint i = 0;
    for(auto it = ptr_data->begin(); it != ptr_data->end() && i < length; ++it, ++i) {
        output << *it;
    }
    return output.str();
}

std::__cxx11::string RandomAccessBinary::get_hex_string(uint byteOffset, uint length)
{
    std::stringstream output;
    output << std::hex << std::noshowbase << std::right << std::setfill('0');
    uint i = 0;
    for(auto it = ptr_data->begin()+byteOffset; it != ptr_data->end() && i < length; ++it, ++i) {
        output << std::setw(2) << static_cast<int>(*it);
    }
    return output.str();
}

std::__cxx11::string RandomAccessBinary::get_hex_string()
{
    std::stringstream output;
    output << std::hex << std::noshowbase << std::right << std::setfill('0');
    for(auto it = ptr_data->begin(); it != ptr_data->end(); ++it) {
        output << std::setw(2) << static_cast<int>(*it);
    }
    return output.str();
}
}
