#ifndef RANDOMACCESSBINARY_H
#define RANDOMACCESSBINARY_H

#include <list>
#include <memory>
#include <vector>
namespace NPC_core {

class RandomAccessBinary
{
public:
    enum class Endianness{
        BIG,
        LITTLE,
        MIDDLE
    };

    RandomAccessBinary();
    RandomAccessBinary(uint possible_packetsize);
    virtual ~RandomAccessBinary();

    void setEndianness(Endianness endianess);
    Endianness getEndianness();

    void clear() { ptr_data->clear(); }

    void setBytes(uint position, const std::vector<uint8_t>& data);
    void setByte(uint position, const uint8_t& data);

    void setBit(uint64_t position, const bool& value);
    void setBit(uint bytePosition, uint bitPosition, const bool& value);
    void setBits(uint64_t position, uint bits, const uint8_t& value);
    void setBits(uint bytePosition, uint bitPosition, uint bits, const uint8_t& value);

    void set_uint16(uint position, const uint16_t& data);
    void set_uint32(uint position, const uint32_t& data);
    void set_uint64(uint position, const uint64_t& data);

    void set_float(uint position, float& data);
    void set_double(uint position, double& data);

    uint8_t
    getByte(uint position);

    const std::vector<uint8_t>
    getBytes(uint position, uint length);

    bool getBit(uint byteOffset, uint bitOffset);
    bool getBit(uint64_t position);
    const std::vector<bool>
    getBits(uint byteOffset, uint8_t bitOffset, uint64_t length);

    const std::vector<bool>
    getBits(uint64_t position, uint64_t length);

    uint16_t get_uint16(uint byteOffset);
    uint32_t get_uint32(uint byteOffset);
    uint64_t get_uint64(uint byteOffset);

    float get_float(uint byteOffset);
    double get_double(uint byteOffset);

    std::string get_string(uint byteOffset, uint length);

    std::string get_hex_string(uint byteOffset, uint length);
    std::string get_hex_string();

    uint64_t size() { return ptr_data->size(); }
private:
    std::unique_ptr<std::vector<uint8_t>> ptr_data;
    Endianness endianness;
};
}
#endif // RANDOMACCESSBINARY_H
