#ifndef EDITORELEMENT_H
#define EDITORELEMENT_H
#include <QString>
#include <QMap>
#include <QList>
#include <list>
#include <string>

enum class ValueTypeShort {
    BIN,OCT,DEC,HEX,TXT
};


class EditorElementData
{
public:

    explicit EditorElementData();
    virtual ~EditorElementData();


    QString getTitle() const;
    void setTitle(const QString &value);
    void setTitle(const std::string &value) {
        this->setTitle(QString::fromStdString(value));
    }

    QString getId() const;
    void setId(const QString &value);
    void setId(const std::string &value){
        this->setId(QString::fromStdString(value));
    }

    QList<QString> getSyntaxes() const;
    void setSyntaxes(const QList<QString> &value);
    void setSyntaxes(const std::list<std::string> &value) {
        QList<QString> newlist;
        for(std::string str : value)
            newlist.append(QString::fromStdString(str));
        this->setSyntaxes(newlist);
    }

    bool isMultiLine() const;
    void setIsMultiLine(bool value);

    ValueTypeShort getType() const;
    void setType(const ValueTypeShort &value);

    QString getRegex() const;
    void setRegex(const QString &value);
    void setRegex(const std::string &value){
        this->setRegex(QString::fromStdString(value));
    }

    QMap<QString, QString> getHints() const;
    void setHint(const QString &key, const QString &value);
    void setHint(const std::string &key, const std::string &value){
        this->setHint(QString::fromStdString(key), QString::fromStdString(value));
    }

    ulong getBytes() const;
    void setBytes(const ulong &value);

    ulong getBits() const;
    void setBits(const ulong &value);

    bool isData() const;
    void setData(bool value);

    bool isNextLayer() const;
    void setNextLayer(bool value);

    bool isOptional() const;
    void setOptional(bool value);

private:
    QString title;
    QString id;
    QList<QString> syntaxes;
    bool multiline;
    ValueTypeShort type;
    ulong bytes;
    ulong bits;
    QString regex;
    QMap<QString, QString> hints;
    bool data;
    bool nextLayer;
    bool optional;

};

#endif // EDITORELEMENT_H
