#include "guidededitorelementview.h"

#include "flowlayout.h"
#include <QLabel>
#include <QRegExp>
#include <QRegExpValidator>
#include <QVBoxLayout>
#include <memory>

#include <iostream>

GuidedEditorElementView::GuidedEditorElementView(const std::shared_ptr<TreeNode<Field>> &ref, QWidget* parent)
    : GuidedEditorElementView(parent)
{
    this->setReference(ref);
}
GuidedEditorElementView::GuidedEditorElementView(QWidget* parent)
    : QWidget(parent)
{
    this->validator = new QRegExpValidator();
    this->layout = new QVBoxLayout(this);
    this->setLayout(layout);
    this->edit = nullptr;
    this->line = nullptr;
    this->title = new QLabel(this);
}

void GuidedEditorElementView::render()
{
    // TODO: add possibility to collapse a subtree
    for (QObject* w : this->children())
        if (w->isWidgetType()) {
            this->layout->removeWidget((QWidget*)w);
        }

    //title
    title->setText(QString::fromStdString(reference->getData()->getName()));
    layout->addWidget(title);

    if (reference->getChildren()->size() == 0) {
        // need global information about showing syntax, and other hints

        if (false ){// reference->getData()->isMultiLine()) {
            layout->addWidget(edit);
        } else {
            layout->addWidget(line);
        }
    } else {
        //create container for subelements
        if (subContainer == nullptr) {
            subContainer = new QWidget(this);
            FlowLayout* flayout = new FlowLayout(subContainer);
            //create and render subelementviews! this is DEPTH_FIRST!!! TODO: make this breadth first!
            for (std::shared_ptr<TreeNode<Field>> node : reference->getChildren().get()) {
                GuidedEditorElementView* view = new GuidedEditorElementView(subContainer);
                view->setReference(node);
                flayout->addWidget(view);
                view->render();
            }
        }
        layout->addWidget(subContainer);
    }
}

void GuidedEditorElementView::inputChanged()
{
    // run regex validation
    if (!validator->regExp().isEmpty()) {
        int pos;
        QValidator::State state;
        if (edit != nullptr) {
            QString data = edit->document()->toPlainText();
            state = validator->validate(data, pos);
            if (state == QValidator::State::Invalid) {
                edit->setTextColor(QColor(0xFF, 0, 0));
            } else if (state == QValidator::State::Intermediate) {
                edit->setTextColor(QColor(0xFF, 0xFF, 0));
            } else {
                line->setStyleSheet("color: #000000");
            }
        }
        if (line != nullptr) {
            QString data = line->text();
            state = validator->validate(data, pos);
            if (state == QValidator::State::Invalid) {
                line->setStyleSheet("color: #FF0000");
            } else if (state == QValidator::State::Intermediate) {
                line->setStyleSheet("color: #FFFF00");
            } else {
                line->setStyleSheet("color: #000000");
            }
        }
    }
}

void GuidedEditorElementView::dataChanged()
{
    //should be reload!
    this->render();
}

void GuidedEditorElementView::nodeChanged()
{
    this->render();
}

std::shared_ptr<TreeNode<Field>> GuidedEditorElementView::getReference() const
{
    return reference;
}

void GuidedEditorElementView::setReference(const std::shared_ptr<TreeNode<Field>>& value)
{
    reference = value;
    /*QRegExp r(reference->getData()->getRegex());
    validator->setRegExp(r);
    if (reference->getData()->isMultiLine()) {
        edit = new QTextEdit(this);
        this->connect(edit, &QTextEdit::textChanged, this, &GuidedEditorElementView::dataChanged);
    } else {
        line = new QLineEdit(this);
        this->connect(line, &QLineEdit::textChanged, this, &GuidedEditorElementView::dataChanged);
    }*/
}
