#include <memory>

#include <iostream>

#include "guidededitorelementview.h"
#include "guidededitorview.h"

GuidedEditorView::GuidedEditorView(QWidget* parent)
    : QWidget(parent)
{
    editorLayout = new FlowLayout(this);
    this->setLayout(editorLayout);
    // tests
    QString style = QString(
        "QWidget {"
        "border-radius: 5px;"
        "background: white }"
        "QLabel {border: 2px dashed black; background: gray; "
        "border-radius: 5px;}"
        "QLineEdit {border: 2px dashed black;background: dark-gray;"
        "border-radius: 5px;}");
    this->setStyleSheet(style);
    this->addElement(nullptr);
}

GuidedEditorView::GuidedEditorView(
        Tree<Field>* tree,
    QWidget* parent)
    : GuidedEditorView(parent)
{
    this->tree = tree;
}

GuidedEditorView::~GuidedEditorView() {}

void GuidedEditorView::addElement(EditorElementData* element)
{
    std::cout << "adding elementview from element " << element << std::endl;
    GuidedEditorElementView* elementView = new GuidedEditorElementView(this);
    this->layout()->addWidget(elementView);
    //this->update();
}

void GuidedEditorView::setTree(Tree<Field>* tree)
{
    this->tree = tree;

}

void GuidedEditorView::treeDataChangedSlot(std::shared_ptr<TreeNode<Field>> node)
{
    // get node view
    auto view = nodeToViewMap[node];
    if(view != nullptr)
        view->dataChanged();
}

void GuidedEditorView::treeChangedSlot(std::shared_ptr<TreeNode<Field>> node)
{
    // get node view
    auto view = nodeToViewMap[node];
    if(view != nullptr)
        view->nodeChanged();
}

void GuidedEditorView::render()
{
    if(tree != nullptr) {
        // root node is invisible!
        for(std::shared_ptr<TreeNode<Field>> node : tree->getRoot()->getChildren()){
            auto view = new GuidedEditorElementView(node, this);
            this->editorLayout->addWidget(view);
            this->nodeToViewMap.insert(node, view);
        }
    }
}
