#ifndef PACKETELEMENT_H
#define PACKETELEMENT_H

#include <cstdint>
#include <string>
#include <list>
#include <memory>


class Field;

typedef
    std::list<std::shared_ptr<Field>>
    FieldList_t;

	/**
	   @brief The ValueType enum
	*/
enum class ValueType { hexBinary, integer, text };

class FieldSyntax
{
public:
	std::string triggerRegex;
	std::string completeRegex;
	std::string example;
};

class FieldProperties
{
/*
length:
        type: integer
      reflength:
        type: string
      data:
        type: boolean
      field: == name
        type: string
      desc:
        type: string
      bitfield:
        type: boolean
      optional:
        type: boolean
      repeatable:
        type: boolean
      nextlayer:
        type: boolean
      reference:
        type: string
      subfields:
        type: array
*/
public:
    std::string protocol;
    std::string name;
    std::string description;
    std::string sizeReferenceField;
    std::string protocolReference;
    bool data, optional, repeatable, nextLayer;
    uint32_t size;
    std::list<std::shared_ptr<FieldSyntax>> syntaxes;
};

class FieldData
{
public:
    std::string data;
    ValueType type;
};

/**
   @brief The Field class
 */

class Field
{
public:
    Field(std::string name, uint32_t sizeInBit, uint16_t layer);
    void		setValue(std::string& value) { this->value = value; this->valueType = ValueType::text; }
	void		setValue(std::string& value, ValueType type);

	const std::string getValue() { return value; }
	uint32_t	getSizeInBit() { return sizeInBit; }
	uint16_t	getLayer() { return layer; }
	const std::string getName() { return name; }
	const std::string getDescription() { return description; }


    const std::shared_ptr<FieldList_t> getNextLayer() { return nextLayer; }
    bool		hasNextLayer() { return nextLayer != nullptr && sizeInBit == 0; }
    void		setNextLayer(std::shared_ptr<FieldList_t>& layerElements) { nextLayer = layerElements; }

	void		addSyntax(std::string minimalRegex, std::string completeRegex, std::string example)
	{
        std::shared_ptr<FieldSyntax> syntax = std::make_shared<FieldSyntax>();
        syntax->triggerRegex = minimalRegex;
        syntax->completeRegex = completeRegex;
        syntax->example = example;
		syntaxes.push_back(syntax);
	}

    std::list<uint8_t> getBytes();

private:
	std::string name;
	uint32_t sizeInBit;
	uint16_t layer;
	std::string value;
	ValueType valueType;

    std::shared_ptr<FieldList_t> nextLayer;

	//additional data
	std::string description;
    std::list<std::shared_ptr<FieldSyntax>> syntaxes;

};

#endif // PACKETELEMENT_H
