#include "DataLoader.h"

#include <boost/filesystem.hpp>
#include <string>

DataLoader::DataLoader()
{

}

DataLoader::DataLoader(std::string directory) : DataLoader()
{
    this->loadProtocolDirectory(directory);
}

void DataLoader::loadProtocolDirectory(std::__cxx11::string directory)
{
    using namespace boost::filesystem;
    // awaiting
    path p(directory);
    if (exists(p)) {
        // load all files in osi* directories
        for(directory_entry& d : directory_iterator(p))
        {
            if(is_directory(d) && d.path().filename().string().find("osi") != std::string::npos)
            {
                path osi = d.path();
                for(directory_entry& od: directory_iterator(p))
                {
                    if(od.path().extension().string().compare(std::string(".yml")) == 0 ) {
                        this->loadSingleProtocol(od.path().string());
                    }
                }
            }
        }
    }
}

void DataLoader::loadSingleProtocol(std::__cxx11::string filename)
{
    YAML::Node node = YAML::LoadFile(filename);
    protocolList.push_back(node);
}

std::shared_ptr<TreeNode<Field>> DataLoader::generateTree(YAML::Node &node)
{
    if(!node["protocol"])
        return nullptr;

    // root node containing the protocol.
    std::shared_ptr<TreeNode<Field>> editorNode = std::shared_ptr<TreeNode<Field>>(new TreeNode<Field>());
    /*/std::shared_ptr<EditorElementData> data = std::shared_ptr<EditorElementData>(new EditorElementData());

    //data->setTitle(node["protocol"]["name"].as<std::string>());
    if(node["protocol"]["longname"])
        data->setHint("subtitle", node["protocol"]["longname"].as<std::string>());
    editorNode->setData(data);*/
    // add children containing the fields
    auto fields = node["protocol"]["fields"];
    for(YAML::Node n : fields) {
        this->genNode(n, editorNode);
    }
    return editorNode;
}

bool DataLoader::getProtocolByName(std::string name, YAML::Node &node)
{
    for(YAML::Node n : protocolList) {
        if(n["protocol"]) {
            if(name.compare(n["protocol"]["name"].as<std::string>()) == 0) {
                node = n;
                return true;
            }
        }
    }
    return false;
}

std::list<std::string> DataLoader::getProtocolNames()
{
    std::list<std::string> names;
    for(YAML::Node node : protocolList) {
        if(node["protocol"]) {
            names.push_back(node["protocol"]["name"].as<std::string>());
        }
    }
    return names;
}

std::shared_ptr<TreeNode<Field>> DataLoader::genNode(YAML::Node &n, std::shared_ptr<TreeNode<Field>> parent)
{

    std::shared_ptr<TreeNode<Field>> newNode = std::shared_ptr<TreeNode<Field>>(new TreeNode<Field>());
    newNode->setParent(parent);
    /*std::shared_ptr<EditorElementData> newData = std::shared_ptr<EditorElementData>(new EditorElementData());
    newData->setTitle(n["field"].as<std::string>());
    if(n["desc"])
        newData->setHint("desc",n["desc"].as<std::string>());
    if(n["bytes"])
        newData->setBytes(n["bytes"].as<ulong>());
    else if (n["bits"])
        newData->setBits(n["bits"].as<ulong>());
    if(n["data"])
        newData->setData(n["data"].as<bool>());
    if(n["optional"])
        newData->setOptional(n["data"].as<bool>());
    if(n["nextlayer"])
        newData->setNextLayer(n["nextlayer"].as<bool>());
    newNode->setData(newData);*/
    if(n["subfields"])
    {
        auto fields = n["subfields"];
        for(YAML::Node subn : fields)
            newNode->addChild(this->genNode(subn, newNode));
    }
    return newNode;
}
